//========================================================================
// GLFW - An OpenGL framework
// File:     enable.c
// Platform: Windows
// Version:  2.3
// Date:     2002.11.23
// Author:   Marcus Geelnard (marcus.geelnard@home.se)
// WWW:      http://hem.passagen.se/opengl/glfw/
//------------------------------------------------------------------------
// Copyright (c) 2002 Marcus Geelnard
//
// GLFW is provided "as-is", without any express or implied warranty,
// without even the implied warranty of fitness for a particular purpose.
// In no event will the author be held liable for any damage araising from
// the use of GLFW.
//
// Permission is granted to anyone to use GLFW for any purpose, including
// commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. You must not claim that you wrote GLFW.
// 2. If you modify any parts of GLFW, it must be clearly stated which
//    modifications have been made, and that those modifications are not
//    the work of the author of GLFW.
// 3. If you use GLFW in a product, an acknowledgment in the product
//    documentation would be appreciated, but is not required.
// 4. This notice may not be removed or altered from any source
//    distribution of GLFW.
//
// Marcus Geelnard
// marcus.geelnard@home.se
//========================================================================

#include "internal.h"


//************************************************************************
//****                  GLFW internal functions                       ****
//************************************************************************

//========================================================================
// _glfwEnableMouseCursor() - Enable (show) mouse cursor
// _glfwDisableMouseCursor() - Disable (hide) mouse cursor
//========================================================================

static void _glfwEnableMouseCursor( void )
{
    // Is GLFW initialized?
    if( !_glfwInitialised || !_glfwWin.Opened || !_glfwWin.MouseLock )
    {
        return;
    }

    // From now on the mouse is unlocked
    _glfwWin.MouseLock = GL_FALSE;

    // Un-capture cursor
    ReleaseCapture();

    // Show cursor
    ShowCursor( TRUE );
}

static void _glfwDisableMouseCursor( void )
{
    POINT pos;

    // Is GLFW initialized?
    if( !_glfwInitialised || !_glfwWin.Opened || _glfwWin.MouseLock )
    {
        return;
    }

    // From now on the mouse is locked
    _glfwWin.MouseLock = GL_TRUE;

    // Hide cursor
    ShowCursor( FALSE );

    // Capture cursor to user window
    SetCapture( _glfwWin.Wnd );

    // Move cursor to the middle of the window
    pos.x = _glfwWin.Width>>1;
    pos.y = _glfwWin.Height>>1;
    ClientToScreen( _glfwWin.Wnd, &pos );
    SetCursorPos( pos.x, pos.y );
}


//========================================================================
// _glfwEnableStickyKeys() - Enable sticky keys
// _glfwDisableStickyKeys() - Disable sticky keys
//========================================================================

static void _glfwEnableStickyKeys( void )
{
    _glfwInput.StickyKeys = 1;
}

static void _glfwDisableStickyKeys( void )
{
    int i;

    _glfwInput.StickyKeys = 0;

    // Release all sticky keys
    for( i = 0; i <= GLFW_KEY_LAST; i ++ )
    {
        if( _glfwInput.Key[ i ] == 2 )
        {
            _glfwInput.Key[ i ] = 0;
        }
    }
}


//========================================================================
// _glfwEnableStickyMouseButtons() - Enable sticky keys
// _glfwDisableStickyMouseButtons() - Disable sticky keys
//========================================================================

static void _glfwEnableStickyMouseButtons( void )
{
    _glfwInput.StickyMouseButtons = 1;
}

static void _glfwDisableStickyMouseButtons( void )
{
    int i;

    _glfwInput.StickyMouseButtons = 0;

    // Release all sticky keys
    for( i = 0; i <= GLFW_MOUSE_BUTTON_LAST; i ++ )
    {
        if( _glfwInput.MouseButton[ i ] == 2 )
        {
            _glfwInput.MouseButton[ i ] = 0;
        }
    }
}


//========================================================================
// _glfwLLKeyboardProc() - Low level keyboard callback function (used to
// disable system keys under Windows NT).
//========================================================================

LRESULT CALLBACK _glfwLLKeyboardProc( int nCode, WPARAM wParam,
    LPARAM lParam )
{
    BOOL syskeys = 0;
    PKBDLLHOOKSTRUCT p;

    // We are only looking for keyboard events - interpret lParam as a
    // pointer to a KBDLLHOOKSTRUCT
    p = (PKBDLLHOOKSTRUCT) lParam;

    // If nCode == HC_ACTION, then we have a keyboard event
    if( nCode == HC_ACTION )
    {
        switch( wParam )
        {
            case WM_KEYDOWN:
            case WM_SYSKEYDOWN:
            case WM_KEYUP:
            case WM_SYSKEYUP:
                // Detect: ALT+TAB, ALT+ESC, ALT+F4, CTRL+ESC,
                // LWIN, RWIN, APPS (mysterious menu key)
                syskeys = ( p->vkCode == VK_TAB &&
                            p->flags & LLKHF_ALTDOWN ) ||
                          ( p->vkCode == VK_ESCAPE &&
                            p->flags & LLKHF_ALTDOWN ) ||
                          ( p->vkCode == VK_F4 &&
                            p->flags & LLKHF_ALTDOWN ) ||
                          ( p->vkCode == VK_ESCAPE &&
                            (GetKeyState(VK_CONTROL) & 0x8000)) ||
                          p->vkCode == VK_LWIN ||
                          p->vkCode == VK_RWIN ||
                          p->vkCode == VK_APPS;
                break;

            default:
                break;
        }
    }

    // Was it a system key combination (e.g. ALT+TAB)?
    if( syskeys )
    {
        // Pass the key event to our window message loop
        if( _glfwWin.Opened )
        {
            PostMessage( _glfwWin.Wnd, (UINT) wParam, p->vkCode, 0 );
        }

        // We've taken care of it - don't let the system know about this
        // key event
        return 1;
    }
    else
    {
        // It's a harmless key press, let the system deal with it
        return CallNextHookEx( NULL, nCode, wParam, lParam );
    }
}

//========================================================================
// _glfwEnableSystemKeys() - Enable system keys
// _glfwDisableSystemKeys() - Disable system keys
//========================================================================

static void _glfwEnableSystemKeys( void )
{
    OSVERSIONINFO osi;
    BOOL bOld;

    if( !_glfwWin.SysKeysDisabled )
    {
        return;
    }

    // Use different methods depending on operating system version
    osi.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
    GetVersionEx( &osi );
    if( osi.dwPlatformId == VER_PLATFORM_WIN32_NT )
    {
        if( _glfwWin.KeyboardHook != NULL )
        {
            UnhookWindowsHookEx( _glfwWin.KeyboardHook );
            _glfwWin.KeyboardHook = NULL;
        }
    }
    else if( osi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS )
    {
        (void) SystemParametersInfo( SPI_SETSCREENSAVERRUNNING, FALSE,
                                     &bOld, 0 );
    }

    // Indicate that system keys are no longer disabled
    _glfwWin.SysKeysDisabled = GL_FALSE;
}

static void _glfwDisableSystemKeys( void )
{
    OSVERSIONINFO osi;
    BOOL bOld;

    if( _glfwWin.SysKeysDisabled )
    {
        return;
    }

    // Use different methods depending on operating system version
    osi.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
    GetVersionEx( &osi );
    if( osi.dwPlatformId == VER_PLATFORM_WIN32_NT )
    {
        // Under Windows NT, install a low level keyboard hook
        _glfwWin.KeyboardHook = SetWindowsHookEx( WH_KEYBOARD_LL,
                                    _glfwLLKeyboardProc,
                                    _glfwWin.Instance,
                                    0 );
    }
    else if( osi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS )
    {
        // Under Windows 95/98/ME, fool Windows that a screensaver
        // is running => prevents ALT+TAB, CTRL+ESC and CTRL+ALT+DEL
        (void) SystemParametersInfo( SPI_SETSCREENSAVERRUNNING, TRUE,
                                     &bOld, 0 );
    }

    // Indicate that system keys are now disabled
    _glfwWin.SysKeysDisabled = GL_TRUE;
}


//========================================================================
// _glfwEnableKeyRepeat() - Enable key repeat
// _glfwDisableKeyRepeat() - Disable key repeat
//========================================================================

static void _glfwEnableKeyRepeat( void )
{
    _glfwInput.KeyRepeat = 1;
}

static void _glfwDisableKeyRepeat( void )
{
    _glfwInput.KeyRepeat = 0;
}



//************************************************************************
//****                    GLFW user functions                         ****
//************************************************************************

//========================================================================
// glfwEnable() - Enable certain GLFW/window/system functions.
//========================================================================

GLFWAPI void GLFWAPIENTRY glfwEnable( int token )
{
    switch( token )
    {
    case GLFW_MOUSE_CURSOR:
        _glfwEnableMouseCursor();
        break;
    case GLFW_STICKY_KEYS:
        _glfwEnableStickyKeys();
        break;
    case GLFW_STICKY_MOUSE_BUTTONS:
        _glfwEnableStickyMouseButtons();
        break;
    case GLFW_SYSTEM_KEYS:
        _glfwEnableSystemKeys();
        break;
    case GLFW_KEY_REPEAT:
        _glfwEnableKeyRepeat();
        break;
    default:
        break;
    }
}


//========================================================================
// glfwDisable() - Disable certain GLFW/window/system functions.
//========================================================================

GLFWAPI void GLFWAPIENTRY glfwDisable( int token )
{
    switch( token )
    {
    case GLFW_MOUSE_CURSOR:
        _glfwDisableMouseCursor();
        break;
    case GLFW_STICKY_KEYS:
        _glfwDisableStickyKeys();
        break;
    case GLFW_STICKY_MOUSE_BUTTONS:
        _glfwDisableStickyMouseButtons();
        break;
    case GLFW_SYSTEM_KEYS:
        _glfwDisableSystemKeys();
        break;
    case GLFW_KEY_REPEAT:
        _glfwDisableKeyRepeat();
        break;
    default:
        break;
    }
}
